<?php

namespace Strig\Image;

class Image
{
    protected $path;
    protected $newFilename;
    protected $output;
    protected $image;
    protected $dirty;

    /**
     * @param string $path
     * @param string $output
     */
    public function __construct(string $path, string $output = null)
    {
        $this->path = $path;
        $this->output = $output ?? dirname($path);

        // output required when using remote image
        // otherwise won't save le image
        // $this->output = (substr($this->output, 0, 4) !== 'http') ?: '';
        // $this->dirty = is_writeable($this->output);

        $this->image = $this->getImagickObject($this->path);
    }


    /**
     * @param int $width
     * @param int $height
     *
     * @return bool
     */
    public function resize(int $width, int $height): bool
    {
        // New filename
        $this->newFilename = $this->generateNewFilename($this->path, $width, $height);

        return $this->image->resizeImage($width, $height, 0, .75);
    }


    /**
     * @param string|null $output
     * @return mixed
     */
    public function save(string $output = null)
    {
        if (isset($output)) {
            $this->output = $output;
        }

        if (!is_dir($this->output)) {
            mkdir($this->output, 0777, true);
        }

        $this->image->setFormat('png');
        $this->image->writeImage($this->newFilename);

        return $this->getImagePath();
    }


    /**
     * @return bool
     */
    public function flush(): bool
    {
        $status = false;

        if ($this->dirty) {
            $this->image->setFormat('png');
            $status = $this->image->writeImage($this->output . '/' . $this->newFilename);
        }

        return $status;
    }

    /**
     * @return string
     */
    public function getImagePath(): string
    {
        return sprintf('%s/%s.png', $this->output, $this->newFilename);
    }

    /**
     * Get image at path/url if exist. Get 1x1 transparent image otherwise.
     *
     * @param string $path
     *
     * @return \Imagick
     */
    protected function getImagickObject(string $path): \Imagick
    {
        try {
            $image = new \Imagick();

            if (!file_exists($this->path)) {
                $image->readImageBlob(base64_decode('iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR42mNkYAAAAAYAAjCB0C8AAAAASUVORK5CYII='));
            } else {
                $image = new \Imagick($path);
            }

            return $image;
        } catch (\ImagickException $e) {
        }

    }

    /**
     *
     *
     * @param string $path
     * @param int $width
     * @param int $height
     *
     * @return string
     */
    protected function generateNewFilename(string $path, int $width, int $height): string
    {
        return sprintf('%s-%dx%d.png', md5($path), $width, $height);
    }
}
